/* blackbox.c  -*- C -*-
 *
 *  Created:   1999/06/02 Jos Romildo Malaquias 
 *                          <romildo@iceb.ufop.br>
 *  Modify: 2000/01/31 Igor Homyakov
 *                          <comandore@mail.ru>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*
 * Functions needed for producing Blackbox configuration
 */

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*
 * Outputs on stdout a blackbox-compatible configuration
 */
static void output_blackbox_level(struct group *root, int level);

/* Return valid menu item  */
char* valid_string (char *str);

void output_blackbox(struct group *root) {
    return output_blackbox_level(root, 0);
}

void output_blackbox_level(struct group *root, int level)
{
    struct item *item;
    int sp;

    if (root == (struct group *)NULL)
        return;

    if (level > 0) 
        for (sp = 0; sp < level; sp++) putchar('\t');
    
    if (level == 0)
	printf("[begin] (%s)\n", valid_string(root->name));
    else
	printf("[submenu] (%s)\n", valid_string(root->name));

    item = root->items;
    while (item->type != 0) {
        if (item->type == ITEM_MENU) {
	    struct group *subgroup = (struct group *)item->data;
            output_blackbox_level(subgroup, level + 1);
	} else if (item->type == ITEM_APP) {
            struct package *app;

	    app = (struct package *)item->data;
	    if (app->name && app->exec && app->__package) {
                for (sp = 0; sp <= level; sp++) putchar('\t');
		printf("[exec] (%s) {", valid_string(app->name));
					
                /* this removes ending '&' that should not be there */
                {
                    char *b;
                    int l;

                    b = app->exec;
                    l = strlen(b);
                    while (l > 0 && b[l - 1] == ' ')
                        l--;
                    if (l > 0 && b[l - 1] == '&')
                        l--;
                    for (; l > 0; l--,b++)
                        putchar(*b);
                    printf("}\n");
                }
	    }
	}
	item++;
    }
    if (level > 0) 
        for (sp = 0; sp < level; sp++) putchar('\t');
    
    printf("[end]\n");
}

/*  
 *  get:	"Blackbox is cool, (small) and [fast]"
 *  return:	"Blackbox is cool, /(small/) and /[fast/]"
 * 
 *  Warning: This function modifies its argument, and can't be used 
 *           with constant strings. 
 */
char*
valid_string (char *str)
{
    char *brackets = "[]()"; 
    char *p, *q; 
    int i, c, k;
    int list[200];
    int sz;
    
    c = -1;
    i = 0;
    sz = strlen (str);
    
    while (i < sz) {	
	if (strchr (brackets, str[i]))
	    list[++c] = i;
	i++;  
    }
    
    if (c < 0) 
	return str;
    
    p = (char*) malloc (sz + c);
    if (p == NULL) {
	fprintf (stderr, "%d@%s: out of memory !\n", 
		 __LINE__-1, __FUNCTION__);
	
	return NULL;
    }
    
    memset (p, 0, sz + c);
    
    q = str;
    k = 0;
    for (i = 0; i <= c + 1; i++) {
	strncat (p, q, list[i] - k);
	
	if (i != c + 1) 
	    strcat (p, "\\");
	
	q = &str[list[i]];
	k = list[i];
    }
    
    free (str);
    return str = p;
}
