/*
 * Copyright (C) 1997 Red Hat Software, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * WMCONFIG.C
 *
 * Core of Wmconfig
 */

#ifdef HAVE_CONFIG_H
 #include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <dirent.h>
#include <pwd.h>
#include <limits.h>
static const char *ProgName = NULL;

/*
Define which Window manager we support
*/

#define DO_FVWM95 1
#define DO_FVWM2 2
#define DO_AFTERSTEP 3
#define DO_MWM 4
#define DO_ICEWM 5
#define DO_WMAKER 6
#define DO_KDE 7
#define DO_DEBUG 8
#define DO_BLACKBOX 9
#define DO_TWM 10
#define DO_OLWM 11
#define DO_FVWM 12
#define DO_MLVWM 13
#define DO_KDE2 14
#define DO_QVWM 15
#define DO_PEKWM 16
#define DO_GOLEM 17

static int output_type 	= 0;
unsigned int flags = 0;
#include "wmconfig.h"
struct group *root_group = NULL;
extern struct package *packages;
extern int num_packages;
static const char *sys_confdir = NULL;
static const char *user_confdir = NULL;
const char *root_menu_name = NULL;
const char *output_directory = NULL;

/*
Generic help function
*/

static void usage(void)
{
 printf("Usage: \t%s [options] [file ...]\nwhere options can be:\n", ProgName);
 printf("  --output\tdebug | fvwm95 | fvwm2 | afterstep | mwm |\n"
  "\t\ticewm | blackbox | wmaker | kde1 | twm | olwm | olvwm\n"
  "\t\tfvwm | mlvwm | kde2 | kde3 | qvwm | pekwm | golem\n"
  "\t\tfluxbox\n");
 printf("\t\t\t(gives the type of output; default is debug)\n");
 printf("  --sysdir\tsys_config_dir\t\t(default: \"%s\")\n", DEFAULT_SYS_CONFDIR?DEFAULT_SYS_CONFDIR:"");
 printf("  --userdir\tuser_config_dir\t\t(default: \"$HOME/%s\")\n", DEFAULT_USER_CONFDIR?DEFAULT_USER_CONFDIR:"");
 printf("  --rootmenu\troot_menu_name\t\t(default: \"%s\")\n", DEFAULT_ROOT_NAME?DEFAULT_ROOT_NAME:"");
 printf("  --outputdir\toutput_directory\t(default: \"%s\")\n", DEFAULT_OUTPUTDIR?"$HOME/"DEFAULT_OUTPUTDIR:"");
 printf("  --flag\tno-icons,no-mini-icons,promote,\n");
 printf("\t\tdirectories,newstyle-directories\t(no defaults)\n");
 printf("  --check-existence\n");
 printf("  --no-icons\n");
 printf("  --no-mini-icons\n");
 printf("  --directories\n");
 printf("  --newstyle-directories\n");
 printf("  --no-promote\n");
 printf("  --no-sysdir\n");
 printf("  --no-userdir\n");
 printf("  --help\n");
 printf("  --version\n");
 exit(-1);
}
/*
Version function
*/
static void version(void)
{
 #ifdef VERSION
  printf("%s version %s\n", ProgName, VERSION);
 #else
  printf("%s (developer release)\n", ProgName);
 #endif
 printf("$Id: wmconfig.c,v 1.24 2000/04/03 22:41:15 gafton Exp $\n");
 exit(0);
}
/*
Read and parse given options
*/
static int parse_options(int argc, char **argv)
{
 int option_index = 0;
 int c;
 static int val = 0;
 static struct option options[] = {
  { "output", required_argument, 0, 'o' },
  { "sysdir", required_argument, 0, 's' },
  { "userdir", required_argument, 0, 'u' },
  { "rootmenu", required_argument, 0, 'r' },
  { "flags", required_argument, 0, 'f' },
  { "help", no_argument, 0, 'h' },
  { "version", no_argument, 0, 'v' },
  { "no-promote", no_argument, &val, FLAG_NO_PROMOTE },
  { "directories", no_argument, &val, FLAG_DIRECTORIES },
  { "newstyle-directories", no_argument, &val, FLAG_NEWSTYLE_DIRECTORIES },
  { "no-icons",	no_argument, &val, FLAG_NO_ICONS },
  { "check-existence", no_argument, &val, FLAG_CHECK_EXISTENCE },
  { "no-mini-icons", no_argument, &val, FLAG_NO_MINI_ICONS },
  { "no-sysdir", no_argument, &val, FLAG_NO_SYSDIR },
  { "no-userdir", no_argument, &val, FLAG_NO_USERDIR },
  { "outputdir", required_argument, 0, 'd'},
  {0, 0, 0, 0 },
 };
 while ((c = getopt_long(argc, argv, "o:s:u:hr:f:d:", options, &option_index)) != EOF) {
  switch (c) {
   case 0:
    flags |= val;
    val = 0;
   break;
   case 'o':
    if (optarg == NULL) {
     usage();
     break;
    }
/*
This makes the options case unsensitive
TWM is the same as twm or TwM

Fvwm Output requested - launch the function
defined below
*/
    if (strncasecmp(optarg, "fvwm", 4) == 0)
     output_type = DO_FVWM;
/*
Fvwm2 Output
*/
    if (strncasecmp(optarg, "fvwm2", 5) == 0) 
     output_type = DO_FVWM2;
    if (strncasecmp(optarg, "fvwm95", 6) == 0) 
     output_type = DO_FVWM95;
    if (strncasecmp(optarg, "afterstep", 9) == 0)
     output_type = DO_AFTERSTEP;
    if (strncasecmp(optarg, "mwm", 3) == 0) 
     output_type = DO_MWM;
    if (strncasecmp(optarg, "wmaker", 6) == 0) 
     output_type = DO_WMAKER;
    if (strncasecmp(optarg, "icewm", 5) == 0)
     output_type = DO_ICEWM;
    if (strncasecmp(optarg, "fluxbox", 7) == 0)
     output_type = DO_BLACKBOX;
    if (strncasecmp(optarg, "blackbox", 8) == 0)
     output_type = DO_BLACKBOX;
    if (strncasecmp(optarg, "kde1", 4) == 0)
     output_type = DO_KDE;
/*
Debug output - something special
*/
    if (strncasecmp(optarg, "debug", 5) == 0) 
     output_type = DO_DEBUG;
    if (strncasecmp(optarg, "twm", 3) == 0) 
     output_type = DO_TWM;
    if (strncasecmp(optarg, "olwm", 4) == 0)
     output_type = DO_OLWM;
    if (strncasecmp(optarg, "olvwm", 5) == 0)
     output_type = DO_OLWM;
    if (strncasecmp(optarg, "mlvwm", 5) == 0)
     output_type = DO_MLVWM;
    if (strncasecmp(optarg, "kde2", 4) == 0)
     output_type = DO_KDE2;
    if (strncasecmp(optarg, "kde3", 4) == 0)
     output_type = DO_KDE2;
    if (strncasecmp(optarg, "qvwm", 4) == 0)
     output_type = DO_QVWM;
    if (strncasecmp(optarg, "pekwm", 5) == 0)
     output_type = DO_PEKWM;
    if (strncasecmp(optarg, "golem", 5) == 0)
     output_type = DO_GOLEM;
   break;
/*
Display help if options are missing
*/
   case 's':
    if (optarg == NULL)
     usage();
     sys_confdir = optarg;
   break;
   case 'u':
    if (optarg == NULL)
     usage();
    user_confdir = optarg;
   break;		
   case 'r':
    if (optarg == NULL)
     usage();
    root_menu_name = optarg;
   break;
/*
Parse the options for icons, menus ...
*/
   case 'f':
    if (strstr(optarg, "no-icons") != NULL)
     flags |= FLAG_NO_ICONS;
    if (strstr(optarg, "check-existence") != NULL)
     flags |= FLAG_CHECK_EXISTENCE;
    if (strstr(optarg, "no-mini-icons") != NULL)
     flags |= FLAG_NO_MINI_ICONS;
    if (strstr(optarg, "directories") != NULL)
     flags |= FLAG_DIRECTORIES;
    if (strstr(optarg, "newstyle-directories") != NULL)
     flags |= FLAG_NEWSTYLE_DIRECTORIES;
    if (strstr(optarg, "no-promote") != NULL)
     flags |= FLAG_NO_PROMOTE;
   break;

/*
Output directory - for KDE ...
*/

   case 'd':
    if (optarg == NULL)
     usage();
     output_directory = optarg;
   break;
   case 'v':
    version();
   break;	       
   case 'h':
   default:
    usage();
   break;
  }
 }
 return 0;
}

/*
Read the users home directory - required for input,
if $HOME/.wmconfig exists and for output (KDE ...)
*/

static char *get_home_dir(void)
{
 char *user;
 char *home = NULL;
 struct passwd *pwd;
 struct stat st;
 int ret;
 user = getlogin();
 setpwent();
 if (user != NULL) {
  pwd = getpwnam(user);
 } else {
  pwd = getpwuid(getuid());
 }

/*
No - do not define /dev/null as home directory :-))
*/

 if (pwd == (struct passwd *)NULL) {
  fprintf(stderr, "Can not identify the user home directory !\n");
  return NULL;
 }

/*
Dont use C:\Documents and Settings\This is my machines Admin that cant
admin my machine because mmc.exe always displays a segfault
as home directory
*/
 if (pwd->pw_dir != NULL) {
  int maxlen;
  maxlen = strlen(pwd->pw_dir)+strlen(user_confdir)+5;
  if (maxlen > PATH_MAX) {
   fprintf(stderr, "user home directory %s/%s is too long !\n", home, user_confdir);
   return NULL;
  }
  home = malloc(maxlen);
  if (home == NULL) {
   fprintf(stderr, "%d@%s: out of memory !\n", __LINE__-1, __FUNCTION__);
   endpwent();
   return NULL;
  }
  snprintf(home, maxlen, "%s/%s",
  pwd->pw_dir?pwd->pw_dir:"", user_confdir);
 } else
  return NULL;
 endpwent();
 ret = stat(home, &st);
 if (ret != 0) {
  return NULL;
 }
 if (!S_ISDIR(st.st_mode)) {
  fprintf(stderr, "%d@%s: %s is not a directory\n", __LINE__-1, __FUNCTION__, home);
  return NULL;
 };
 return home;
}

/*
Main function
*/

int main (int argc, char **argv)
{
 int ret;
 char *home;
 int have_files = 0;
 ProgName = argv[0];
 if (parse_options(argc, argv) != 0) {
  fprintf(stderr, "Error parsing program options\n");
  exit(-2);
 }

/*
 Output option is required
*/

 if (!output_type) {
  fprintf(stderr, "Don't know what configuration to output...\n");
  usage();
  exit (99);
 }

/*
 If other options fail, try the default values
*/

 if (sys_confdir == NULL)
  sys_confdir = DEFAULT_SYS_CONFDIR;
 if (user_confdir == NULL)
  user_confdir = DEFAULT_USER_CONFDIR;
 if (root_menu_name == NULL)
  root_menu_name = DEFAULT_ROOT_NAME;
 init_packages();
 if (!is_set(NO_SYSDIR)) {
  ret = parse_files(sys_confdir);
  if (ret != PARSE_OK)
   fprintf(stderr, "Parsing system files error: %s\n",
  parse_errstr(ret));
 }
 
/*
Check if everything is ok with the user files
*/

 if (!is_set(NO_USERDIR)) {
  home = get_home_dir();
  if (home != NULL) {
   ret = parse_files(home);
   if (ret != PARSE_OK) 
    fprintf(stderr, "Parsing user files error: %s\n",
   parse_errstr(ret));
   free(home);
  }
 }

/*
 File failed - permissions or anything else is wrong
*/

 while (argc > optind) {
  have_files ++;
  ret = read_file(argv[optind]);
  if (ret != 0)
   fprintf(stderr, "\tReading file \"%s\" failed !\n", argv[optind]);
  optind++;
 }
 
/*
 Cant find any input files in home directory and the
 system directory
*/

 if (is_set(NO_SYSDIR) && is_set(NO_USERDIR) && !have_files) {
  fprintf(stderr, "No input files were parsed.\n");
  exit(-1);
 }
 setbuf(stdout, NULL);
 if (build_root() != 0) {
  fprintf(stderr, "package configuration error.\n");
  exit(-3);
 }

/*
 the output functions - every function calls
 the $whatever$ file
*/

 switch (output_type) {
  case DO_FVWM95:
   output_fvwm95(root_group);
  break;
  case DO_FVWM2:
   output_fvwm2(root_group);
  break;

/*
 Afterstep directory output requested
 or menu output
*/

  case DO_AFTERSTEP:
   if (is_set(DIRECTORIES) || is_set(NEWSTYLE_DIRECTORIES))
    output_afterstep_dir(root_group);
   else
    output_afterstep(root_group);
  break;
  case DO_MWM:
   output_mwm(root_group);
  break;
  case DO_WMAKER:
   output_wmaker(root_group);
  break;
  case DO_ICEWM:
   output_icewm(root_group);
  break;
  case DO_BLACKBOX:
   output_blackbox(root_group);
  break;
  case DO_KDE:
   output_kde(root_group);
  break;
  case DO_TWM:
   output_twm(root_group);
  break;
  case DO_OLWM:
   output_olwm(root_group);
  break;
  case DO_FVWM:
   output_fvwm(root_group);
  break;
  case DO_MLVWM:
   output_mlvwm(root_group);
  break;
  case DO_KDE2:
   output_kde2(root_group);
  break;
  case DO_QVWM:
   output_qvwm(root_group);
  break;
  case DO_PEKWM:
   output_pekwm(root_group);
  break;
  case DO_GOLEM:
   output_golem(root_group);
  break;
  case DO_DEBUG:
  default:    
   output_debug(root_group);
 }
 return 0;
}
